# Defers the initialization of an instance until a method is called. 
# 
# Standard Usage: 
# 
# Initialization: 
#
# instance = Defer.defer {
#  instance.new
#  instance.do_something
# }
#
# Runtime:
#
# instance.do_another_thing # starts the initialization here, not above
class Defer
   
  # Initializes defer on a block of code
  def self.defer(&block)
    Defer.new(&block)
  end 
   
  def initialize(&block)
    @deferred_init = block
    @semaphore = Mutex.new
  end
   
  # Gets an instance of the object, calling the deferred method if needed
  def get_instance
    @semaphore.synchronize do 
      if @object.nil?
        @object = @deferred_init.call
      else 
        @object
      end
   end
  end
   
   # After initialization is done, defer passes the arguments through 
   #  this block to ensure that everything is completed as required
   def method_missing(method, *args, &block)
    object = get_instance
    object.send(method, *args, &block);
   end
end